package com.sbh.background_locations

import android.app.Activity
import android.app.Application
import android.os.Bundle
import com.sbh.background_locations.callbacks.LocationCallback
import com.sbh.background_locations.models.SBHLocation
import com.sbh.background_locations.streams.LocationStreamHandler
import com.sbh.background_locations.streams.MotionChangeStreamHandler
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.flutter.view.FlutterNativeView

class BackgroundLocationsPlugin() : MethodCallHandler, Application.ActivityLifecycleCallbacks, PluginRegistry.ViewDestroyListener {
    private lateinit var mActivity: Activity
    private lateinit var mMethodChannel: MethodChannel
    private lateinit var mRegistrar: Registrar

    companion object {
        const val TAG: String = "BackgroundLocationsPlugin"
        const val PLUGIN_ID: String = "com.sbh/background_locations"
        private const val METHOD_CHANNEL_NAME: String = "$PLUGIN_ID/methods"

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val methodChannel = MethodChannel(registrar.messenger(), METHOD_CHANNEL_NAME)
            val plugin = BackgroundLocationsPlugin(registrar, methodChannel)
            methodChannel.setMethodCallHandler(plugin)
            registrar.addViewDestroyListener(plugin)
        }
    }

    constructor(registrar: Registrar, channel: MethodChannel) : this() {
        this.mRegistrar = registrar
        this.mActivity = registrar.activity()
        this.mMethodChannel = channel

        this.mRegistrar.addRequestPermissionsResultListener { requestCode, permissions, grantResults ->
            BackgroundLocation.getInstance(mActivity).onRequestPermissionsResult(requestCode, permissions, grantResults)
        }

        LocationStreamHandler().register(registrar)
        MotionChangeStreamHandler().register(registrar)
    }


    override fun onMethodCall(call: MethodCall, result: Result) {
        when {
            call.method == "getCurrentPosition" -> {
                getCurrentPosition(result)
            }
            call.method == "getOdometer" -> {
                getOdometer(result)
            }
            call.method == "setOdometer" -> {
                setOdometer(call.arguments as Double, result)
            }
            call.method == "stop" -> {
                BackgroundLocation.getInstance(mActivity).stop()
            }
            call.method == "start" -> {
                BackgroundLocation.getInstance(mActivity).start()
            }
            else -> result.notImplemented()
        }
    }

    override fun onActivityPaused(activity: Activity?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onActivityResumed(activity: Activity?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onActivityStarted(activity: Activity?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onActivityDestroyed(activity: Activity?) {
        if (mRegistrar.activity() == null) {
            BackgroundLocation.getInstance(mActivity).onActivityDestroy()
            mActivity.application.unregisterActivityLifecycleCallbacks(this)
        }
    }

    override fun onActivitySaveInstanceState(activity: Activity?, outState: Bundle?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onActivityStopped(activity: Activity?) {
//        val config = TSConfig.getInstance(activity)
//        if (config.getEnabled() && config.getEnableHeadless() && !config.getStopOnTerminate()) {
//            TSScheduleManager.getInstance(activity).oneShot(TerminateEvent.ACTION, 10000)
//        }
    }

    override fun onActivityCreated(activity: Activity?, savedInstanceState: Bundle?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onViewDestroy(p0: FlutterNativeView?): Boolean {
        return true
    }

    private fun getOdometer(result: Result) {
        result.success(BackgroundLocation.getInstance(mActivity).getOdometer())
    }

    private fun setOdometer(odometer: Double, result: Result) {
        BackgroundLocation.getInstance(mActivity).setOdometer(odometer, object : LocationCallback {
            override fun onLocation(location: SBHLocation) {
                result.success(location.toMap())
            }

            override fun onError(error: Int) {
                result.error(error.toString(), null, null)
            }
        })
    }

    private fun getCurrentPosition(result: Result) {
        BackgroundLocation.getInstance(mActivity).getCurrentPosition(object : LocationCallback {
            override fun onLocation(location: SBHLocation) {
                result.success(location.toMap())
            }

            override fun onError(error: Int) {
                result.error(error.toString(), "unable to fetch location", null)
            }
        })
    }
}
